function plugindef()
	-- This function and the 'finaleplugin' namespace
	-- are both reserved for the plug-in definition.
	finaleplugin.Author = "Chris"
	finaleplugin.CategoryTags = "Note, Spacing"
	return "- Centrer les Rondes - par Chris", "Centrer les Rondes", ""
end

-- MusicStartPos : x à partir du côté de la page (dépend de systemscaling uniquement) - après les clefs...
-- LeftEdge :  x à partir du côté de la page (dépend de systemscaling uniquement) - avant les clefs...
-- SpaceBefore, SpaceAfter : valeurs fixes - ne dépend pas du scaling
-- MusicWidth : (dépend de systemscaling) = wMesure - SpaceBefore(global) - SpaceAfter(global) - SpaceBefore(mesure) - SpaceAfter(mesure)

local function createString(texte)
	local str=finale.FCString()
	str.LuaString = texte
	return str
end

local function createComboTexts()
	local str = finale.FCString()
	str.LuaString = "Positionnement manuel - Selon l'indication de mesure" .. "|" ..
	"Positionnement manuel - Manuellement(par glisser)" .. "|" ..
	"Diagramme des temps"
	return str:CreateParsedStrings("|")
end

local function getSpaceBeforeOneMeasure(mesure)
	local m = finale.FCMeasure()
	if m:Load(mesure) then
		return m.SpaceBefore
	end
	return 0
end

local function setModePositionNotes(mesure,mode)
	local m = finale.FCMeasure()
	if m:Load(mesure) then
		m.PositioningNotesMode = mode
		m:Save()
	end
end

local function afficheDialog()
	-- boite de dialogue
	local cw = finale.FCCustomWindow()
	cw:SetTitle(createString(plugindef()))
	cw:SetClientWidth(250)
	cw:SetClientHeight(50)
	cw:CreateOkButton() 
	cw:CreateCancelButton()

	-- texte d'info de la combobox
	local textInfo = cw:CreateStatic(0,12)
	textInfo:SetText(createString("Méthode :"))

	-- combobox
	local combo = cw:CreatePopup(40,10)
	combo:SetWidth(210)
	combo:SetStrings(createComboTexts())
	combo:SetSelectedItem(0)

	local ret = cw:ExecuteModal(nil)

	return ret, combo:GetSelectedItem()
end

local function methodManualPositionning(musicregion,distanceprefs,mode)
	for noteentry in eachentrysaved(musicregion) do

		if  noteentry.Duration == finale.WHOLE_NOTE and noteentry:IsNote()  then
			local cell = finale.FCCell(noteentry.Measure, noteentry.Staff)
			local ts = cell:GetTimeSignature()

			-- seulement pour les mesures faisant la lingueur d'une ronde
			if  ts.Beats *  ts.BeatDuration == 4096 then
				setModePositionNotes(noteentry.Measure,mode)

				local cm = cell:CreateCellMetrics()
				local spaceBeforeTotal = distanceprefs.SpaceBefore + getSpaceBeforeOneMeasure(noteentry.Measure)

				local wRonde = noteentry:CalcWidestNoteheadWidth () -- calcul de la largeur de la ronde
				local xRonde =  (cm.Width / 2) - (wRonde / 2) - spaceBeforeTotal

				if mode == finale.POSITIONING_MANUALLY then
					local factor = 600 / cm.MusicWidth
					xRonde = xRonde * factor
				end

				noteentry.ManualPosition = xRonde * cm.SystemScaling / cm.StaffScaling
			end
		end
	end
end

local function methodBeatChart(musicregion,distanceprefs)
	finenv.UI():MenuCommand(finale.MENUCMD_NOTESPACING)  -- pour initialiser les beatcharts

	for noteentry in eachentrysaved(musicregion) do

		if  noteentry.Duration == finale.WHOLE_NOTE and noteentry:IsNote()  then
			local cell = finale.FCCell(noteentry.Measure, noteentry.Staff)
			local ts = cell:GetTimeSignature()

			-- seulement pour les mesures faisant la longueur d'une ronde
			if  ts.Beats *  ts.BeatDuration == 4096 then
				noteentry.ManualPosition = 0    -- supprimer un éventuel positionnement manuel

				local cm = cell:CreateCellMetrics()
				local beatchartelements = finale.FCBeatChartElements()
				beatchartelements:LoadAllForItem(noteentry.Measure)

				-- calcul de la largeur de la ronde
				local wRonde = noteentry:CalcWidestNoteheadWidth ()

				-- spacebefore total (global + mesure)
				local spaceBefore = distanceprefs.SpaceBefore + getSpaceBeforeOneMeasure(noteentry.Measure)

				-- coordonnée de la note (en EVPU) à partir du début du Beat-Chart (après SpaceBefore)
				local xRonde = (cm.Width / 2) - (wRonde / 2) - spaceBefore

				local fullspan
				for bce in each(beatchartelements) do
					if bce:GetInitElement() then -- 1° élément (init)
						fullspan=bce.FullSpan
					else
						bce.MinimumPosition =0   -- avec les altérations il est différent de 0
						bce.HorizontalPosition= fullspan * xRonde/ cm.MusicWidth
						bce:Save()
					end
				end
			end
		end
	end

	finenv.UI():RedrawDocument() 
end

local function main()
	local BTOK = 1      -- constante OK boite de dialogue

	local musicregion = finale.FCMusicRegion()
	musicregion:SetCurrentSelection()
	if musicregion:IsEmpty() then
		finenv.UI():AlertInfo("Veuillez sélectionner des mesures",plugindef())
		return
	end

	local ret, mode = afficheDialog()
	if ret ~= BTOK then return end

	local distanceprefs = finale.FCDistancePrefs()
	distanceprefs:Load(1)

	if mode == 0 then   -- positionnement manuel (Time Signature)
		methodManualPositionning(musicregion,distanceprefs,finale.POSITIONING_TIMESIG)
	elseif mode == 1 then    -- positionnement manuel (Manual positionning)
		methodManualPositionning(musicregion,distanceprefs,finale.POSITIONING_MANUALLY)
	else    -- beat chart
		methodBeatChart(musicregion,distanceprefs)
	end
end

-- DEBUT DU PROGRAMME
main()